### Calculating the outbreak size distribution
install.packages("tidyverse")
install.packages("ggplot2")

library(tidyverse)
library(ggplot2)

##Simulate the SIR model
SIR.model <- function (x, params, nstep) { #function to simulate stochastic SIR
  
  SIR.onestep <- function (x, params) { #function to calculate one step of stochastic SIR
    X <- x[2] #local variable for susceptibles
    Y <- x[3] #local variable for infecteds
    Z <- x[4] #local variable for recovereds
    N <- X+Y+Z #total population size (subject to demographic change)
    with( #use with as in deterministic model to simplify code
      as.list(params),
      {
        rates <- c(mu*N, beta*X*Y/N + eta, mu*X, mu*Y, gamma*Y, mu*Z)
        changes <- matrix(c( 1, 0, 0, 
                            -1, 1, 0,
                            -1, 0, 0,
                            0,-1, 0,
                            0,-1, 1,
                            0, 0,-1),
                          ncol=3, byrow=TRUE)
        tau <- rexp(n=1,rate=sum(rates)) # exponential waiting time
        U <- runif(1) #uniform random deviate
        m <- min(which(cumsum(rates)>=U*sum(rates)))
        x <- x[2:4] + changes[m,]
        return(out <- c(tau, x))
        }
      )
    }
  
  output <- array(dim=c(nstep+1,4)) #set up array to store results
  colnames(output) <- c("time","X","Y","Z") #name variables
  output[1,] <- x #first record of output is initial condition
  for (k in 1:nstep) { #iterate for nstep steps
    output[k+1,] <- x <- SIR.onestep(x,params)
    }
  output #return output
}


##Calculate epidemic size
epidemic_size <- function(data){
  i = 2
  l = length(data$Time)
  epi_data <- data.frame(matrix(ncol = 2, nrow = 0)) %>%
    rename("start_time" = X1,"size" = X2)

  while(i < l){
    if(data$Infected[i] !=0){
      t_epidemic = data$Time[i]
      epidemic_size = 0
      while(data$Infected[i] !=0 && i < l){
        if(data$Infected[i] - data$Infected[i-1] == 1) epidemic_size = epidemic_size + 1
        i = i + 1
      }
      epi_data <- rbind(epi_data, list("start_time" = t_epidemic,"size" =epidemic_size) )
    }
    else i = i + 1
  }
  epi_data <- arrange(epi_data,size) %>%
    mutate(number_greater = length(epi_data$size) -as.numeric(rownames(epi_data)))
  return(epi_data)
}


##Plot Ootbreak size distribution:
plot_outbreak_size <- function(data, binwidth = 30, ylim = c(0,1)){
  ggplot(data = data)+
    geom_histogram(mapping = aes(x=size, y = ..density..), binwidth = binwidth) +
    ylab("Probability")+
    xlab("Outbreak size")+
    ylim(ylim)
}

## Plot number of outbreaks larger than s
plot_number_greater_than_s <- function(outbreak_size_data, xlim = c(1,100)){
  epi_size_cumulative <- outbreak_size_data[!duplicated(outbreak_size_data$size), c("size","number_greater")]
  ggplot(data = epi_size_cumulative, aes(x=size,y= number_greater )) +
    geom_line() +
    geom_point() +
    scale_x_log10(limits=xlim)+
    scale_y_log10() +
    ylab("Number of outbreaks larger than s")+
    xlab("s")
}

##Epidemic simulator
simulate_epidemics <- function(xstart,params,nstep,nsims){
  
  #s <- Sys.time()
  data <- data.frame(matrix(ncol = 5, nrow = 0))
  colnames(data) <- c("time","X","Y","Z","sim")
  for (k in 1:nsims) { #simulate nsims times
    data <- rbind(data, cbind(as.data.frame(SIR.model(xstart,params,nstep)), "sim"=factor(rep(k,nstep+1))))
  }
  
  data <- data %>% 
    group_by(sim) %>%
    mutate(time = cumsum(time)) %>%
    rename("Time" = time, "Susceptible" = X, "Infected"=Y, "Recovered"=Z)
  #print(Sys.time()-s)
  return(data)
}


#Simulation parameters

set.seed(38495) #set seed
nsims <- 10 #number of simulations
pop.size <- 2500 #total population size
Y0 <- 8 #initial number infected
X0 <- pop.size-Y0 #initial number susceptible 
nstep <- 5000 #number of events to simulate
xstart <- c(time=0,X=X0,Y=Y0,Z=pop.size-X0-Y0) #initial conditions
params <- list(mu=1.0/70.0,beta=420,gamma=365/13, eta = 0) #parameters
#data <- vector(mode='list',length=nsims) #initialize list to store the output
R0 <- params$beta/(params$mu + params$gamma)

data <- simulate_epidemics(xstart,params,nstep,nsims)



#Plot time series data
ggplot(data = data[as.numeric(data$sim)<10,])+
  geom_line(mapping = aes(x=Time,y= Infected, color=sim )) + xlim(c(0,0.25))

#Calculate outbreak size distribution and plot
epi_data <- epidemic_size(data)
plot_outbreak_size(epi_data, binwidth = 10, ylim=c(0,1))

#Plot number of outbreaks larger than s
plot_number_greater_than_s (epi_data)


#Read in saved epidemic size data, nstep <- 1250000
epi_data_save <- read.csv("./epidemic_size_data.csv")
plot_outbreak_size(epi_data_save, binwidth = 100, ylim=c(0,0.009))



#Plot of Fraction infectious vs R0:
a <- seq(0,3,0.01)
b <- 10*pop.size*params$mu/params$gamma*(1-1/a)
new_df <- data.frame(a,b) 
new_df[new_df$a < 1,"b"] = 0
ggplot(data = new_df, aes(x = a, y =b)) + 
  geom_line(colour="red", size=1.) + xlab(expression("R"["0"])) + ylab("Fraction infectious")

#Plot of relaxation time
a <- seq(0,3,0.01)
b <- 1/abs(1-a)
new_df <- data.frame(a,b) 
#new_df[new_df$a < 1,"b"] = 0
ggplot(data = new_df, aes(x = a, y =b)) + 
  geom_line(colour="red", size=1.) + xlab(expression("R"["0"])) + ylab("Relaxation time")


